import { dataSource } from "../data-source";
import { userPermission } from "../entity/user_permission";
import InternalException from "../exceptions/internalException";

export default abstract class UserPermissionService {
  /**
   * @description get permission by user
   * @param userId number
   * @returns {Promise<Array<userPermission>>}
   */
  static async getByUser(userId: number): Promise<Array<userPermission>> {
    return await dataSource
      .getRepository(userPermission)
      .createQueryBuilder("permission")
      .where("permission.userId = :userId", { userId: userId })
      .getMany()
      .then((res) => {
        return res;
      })
      .catch((err) => {
        throw new InternalException("permission not found by user", err);
      });
  }
}
